@@ -1,5 +1,5 @@
 /*
- * $Id: msqltcl.c,v 1.50 1995/06/06 14:25:25 hs Rel $
+ * $Id: msqltcl.c,v 1.99 1996/09/27 20:53:09 hs Rel $
  *
  * MSQL interface to Tcl
  *
@@ -8,7 +8,7 @@
  */
 
 /*
- * Copyright (c) 1994, 1995 Hakan Soderstrom and Tom Poindexter
+ * Copyright (c) 1994-1996 Hakan Soderstrom and Tom Poindexter
  * 
  * Permission to use, copy, modify, distribute, and sell this software
  * and its documentation for any purpose is hereby granted without fee,
@@ -64,11 +64,15 @@ static char *MsqlStatusArr = "msqlstatus";
 #define MSQL_STATUS_CMD  "command"
 #define MSQL_STATUS_MSG  "message"
 #define MSQL_STATUS_NULLV  "nullvalue"
+#define MSQL_STATUS_VERSION  "version"
 
 /* C variable corresponding to msqlstatus(nullvalue) */
 static char* MsqlNullvalue = NULL ;
 #define MSQL_NULLV_INIT ""
 
+/* msqltcl version number. */
+static char* MsqlTclVersion = NULL;
+
 /* Options to the 'info', 'result', 'col' combo commands. */
      
 static char* MsqlDbOpt[] =
@@ -202,12 +206,10 @@ msql_reassemble ()
  */
 
 static int
-msql_posix_confl (syscall)
-     char* syscall;
+msql_posix_confl (char* syscall)
 {
   char buf[MSQL_SMALL_SIZE];
-  char* strerror () ;
-  char* msg = strerror (errno) ;
+  char* msg = Tcl_ErrnoMsg (errno);
 
   sprintf(buf, "%d", errno) ;
   Tcl_SetVar2 (saved_interp, MsqlStatusArr, MSQL_STATUS_CODE, buf,
@@ -230,8 +232,7 @@ msql_posix_confl (syscall)
  */
 
 static int
-msql_prim_confl (msg)
-     char* msg ;
+msql_prim_confl (char* msg)
 {
   Tcl_SetVar2 (saved_interp, MsqlStatusArr, MSQL_STATUS_CODE, "-1",
 	       TCL_GLOBAL_ONLY);
@@ -272,8 +273,7 @@ msql_server_confl ()
  * RETURN: MsqlHandle index number or -1 on error.
  */
 static int
-get_handle_plain (handle) 
-     char *handle;
+get_handle_plain (char* handle) 
 {
   int hi ;
   char* hp = handle ;
@@ -294,8 +294,7 @@ get_handle_plain (handle)
  * RETURN: MsqlHandle index number or -1 on error.
  */
 static int
-get_handle_conn (handle) 
-     char *handle;
+get_handle_conn (char* handle) 
 {
   int hi ;
   char* hp = handle ;
@@ -323,8 +322,7 @@ get_handle_conn (handle)
  * RETURN: MsqlHandle index number or -1 on error.
  */
 static int
-get_handle_db (handle) 
-     char *handle;
+get_handle_db (char* handle) 
 {
   int hi ;
   char* hp = handle ;
@@ -359,8 +357,7 @@ get_handle_db (handle)
  */
 
 static int
-get_handle_res (handle) 
-     char *handle;
+get_handle_res (char* handle) 
 {
   int hi ;
   char* hp = handle ;
@@ -385,7 +382,7 @@ get_handle_res (handle)
 
   if (MsqlHandle[hi].result == NULL)
     {
-      msql_prim_confl ("no result pending") ; /*  */
+      msql_prim_confl ("no result pending") ;
       return -1 ;
     }
   else
@@ -416,15 +413,14 @@ handle_init ()
 
 /*
  *----------------------------------------------------------------------
- * clear_msg
+ * msql_clear_msg
  *
  * Clears all error and message elements in the global array variable.
  *
  */
 
 static void
-clear_msg(interp)
-    Tcl_Interp *interp;
+msql_clear_msg(Tcl_Interp* interp)
 {
     Tcl_SetVar2(interp, MsqlStatusArr, MSQL_STATUS_CODE, "0", TCL_GLOBAL_ONLY);
     Tcl_SetVar2(interp, MsqlStatusArr, MSQL_STATUS_CMD, "", TCL_GLOBAL_ONLY);
@@ -440,6 +436,7 @@ clear_msg(interp)
  * having conflict handling.
  * 'req_args' must be the required number of arguments for the command,
  * including the command word.
+ * 'check' may be a pointer to a check function, or NULL.
  * 'usage_msg' must be a usage message, leaving out the command name.
  * Checks the handle assumed to be present in argv[1] if 'check' is not NULL.
  * RETURNS: Handle index or -1 on failure.
@@ -448,13 +445,8 @@ clear_msg(interp)
  */
 
 static int
-msql_prologue (interp, argc, argv, req_args, check, usage_msg)
-     Tcl_Interp *interp;
-     int         argc;
-     char      **argv;
-     int         req_args;
-     char       *usage_msg;
-     int (*check) () ; /* Pointer to function for checking the handle. */
+msql_prologue (Tcl_Interp* interp, int argc, char** argv, int req_args,
+	       int (*check) (), char* usage_msg)
 {
   char buf[MSQL_BUFF_SIZE];
   int hand = 0;
@@ -488,6 +480,33 @@ msql_prologue (interp, argc, argv, req_args, check, usage_msg)
 }
 
 
+/*
+ *----------------------------------------------------------------------
+ * msql_version
+ *
+ * RETURNS the msqltcl version number.
+ *
+ */
+
+static char*
+msql_version ()
+{
+  static char buf[] = "$Revision: 1.99 $";
+  char* ch1;
+  char* ch2;
+
+  /* Find first digit. */
+  for (ch1 = buf; *ch1 < '0' || '9' < *ch1; ++ch1);
+  /* Move digits and dots to the beginning of the string. */
+  for (ch2 = buf; ('0' <= *ch1 && *ch1 <= '9') || *ch1 == '.';
+       *ch2++ = *ch1++);
+  /* Terminate the string. */
+  *ch2 = '\0';
+  
+  return buf;
+}
+
+
 /*
  *----------------------------------------------------------------------
  * msql_colinfo
@@ -500,10 +519,7 @@ msql_prologue (interp, argc, argv, req_args, check, usage_msg)
  */
 
 static int
-msql_colinfo (interp, fld, keyw)
-     Tcl_Interp  *interp;
-     m_field* fld ;
-     char* keyw ;
+msql_colinfo (Tcl_Interp* interp, m_field* fld, char* keyw)
 {
   char buf[MSQL_SMALL_SIZE];
   char keybuf[MSQL_SMALL_SIZE];
@@ -588,8 +604,7 @@ msql_colinfo (interp, fld, keyw)
  */
 
 void
-Msqltcl_Kill (clientData)
-    ClientData clientData;
+Msqltcl_Kill (ClientData clientData)
 {
   int i ;
 
@@ -606,17 +621,14 @@ Msqltcl_Kill (clientData)
 /*
  *----------------------------------------------------------------------
  * Msqltcl_Init
- * Perform all initialization for the MSQL to Tcl interface.
+ * Perform all initialization for the mSQL to Tcl interface.
  * Adds additional commands to interp, creates message array, initializes
  * all handles.
  *
- * A call to Msqltcl_Init should exist in Tcl_CreateInterp or
- * Tcl_CreateExtendedInterp.
  */
 
 int
-Msqltcl_Init (interp)
-    Tcl_Interp *interp;
+Msqltcl_Init (Tcl_Interp* interp)
 {
   int i;
   char nbuf[MSQL_SMALL_SIZE];
@@ -656,10 +668,10 @@ Msqltcl_Init (interp)
 		     Msqltcl_Kill);
 
   /* Initialize msqlstatus global array. */
-  clear_msg(interp);
+  msql_clear_msg (interp);
 
   /* Link the null value element to the corresponding C variable. */
-  if ((MsqlNullvalue = (char*)malloc (12)) == NULL)
+  if ((MsqlNullvalue = (char*)ckalloc (12)) == NULL)
     {
       fprintf (stderr, "*** msqltcl: out of memory\n") ;
       return TCL_ERROR ;
@@ -668,16 +680,25 @@ Msqltcl_Init (interp)
   (void)sprintf (nbuf, "%s(%s)", MsqlStatusArr, MSQL_STATUS_NULLV) ;
   Tcl_LinkVar (interp, nbuf, (char*)&MsqlNullvalue, TCL_LINK_STRING) ;
 
+  /* Set the version string. */
+  MsqlTclVersion = msql_version ();
+
+  /* Link the version variable read-only. */
+  (void)sprintf (nbuf, "%s(%s)", MsqlStatusArr, MSQL_STATUS_VERSION);
+  (void)Tcl_LinkVar (interp, nbuf, (char*)&MsqlTclVersion,
+		     TCL_LINK_STRING | TCL_LINK_READ_ONLY);
+
   /* A little sanity check.
    * If this message appears you must change the source code and recompile.
    */
-  if (strlen (MsqlHandlePrefix) == MSQL_HPREFIX_LEN)
-    return TCL_OK;
-  else
+  if (strlen (MsqlHandlePrefix) != MSQL_HPREFIX_LEN)
     {
       fprintf (stderr, "*** msqltcl (msqltcl.c): handle prefix inconsistency!\n") ;
       return TCL_ERROR ;
     }
+
+  /* Everything seems OK so far; time to make ourselves noticed. */
+  return Tcl_PkgProvide (interp, "msqltcl", MsqlTclVersion);
 }
 
 
@@ -695,11 +716,8 @@ Msqltcl_Init (interp)
  */
 
 int
-Msqltcl_Connect (clientData, interp, argc, argv)
-     ClientData   clientData;
-     Tcl_Interp  *interp;
-     int          argc;
-     char       **argv;
+Msqltcl_Connect (ClientData clientData, Tcl_Interp* interp,
+		 int argc, char** argv)
 {
   int        hand = -1;
   int        i;
@@ -765,11 +783,8 @@ Msqltcl_Connect (clientData, interp, argc, argv)
  *	Sets current database to dbname.
  */
 
-Msqltcl_Use (clientData, interp, argc, argv)
-     ClientData   clientData;
-     Tcl_Interp  *interp;
-     int          argc;
-     char       **argv;
+Msqltcl_Use (ClientData clientData, Tcl_Interp* interp,
+	     int argc, char** argv)
 {
   int hand;
   
@@ -801,11 +816,8 @@ Msqltcl_Use (clientData, interp, argc, argv)
  *    Stores new results.
  */
 
-Msqltcl_Sel (clientData, interp, argc, argv)
-     ClientData   clientData;
-     Tcl_Interp  *interp;
-     int          argc;
-     char       **argv;
+Msqltcl_Sel (ClientData clientData, Tcl_Interp* interp,
+	     int argc, char** argv)
 {
   int     hand;
   
@@ -851,11 +863,8 @@ Msqltcl_Sel (clientData, interp, argc, argv)
  */
 
 int
-Msqltcl_Exec (clientData, interp, argc, argv)
-     ClientData   clientData;
-     Tcl_Interp  *interp;
-     int          argc;
-     char       **argv;
+Msqltcl_Exec (ClientData clientData, Tcl_Interp* interp,
+	      int argc, char** argv)
 {
   int     hand;
   
@@ -888,11 +897,8 @@ Msqltcl_Exec (clientData, interp, argc, argv)
  *	next row from pending results as tcl list, or null list.
  */
 
-Msqltcl_Next (clientData, interp, argc, argv)
-     ClientData   clientData;
-     Tcl_Interp  *interp;
-     int          argc;
-     char       **argv;
+Msqltcl_Next (ClientData clientData, Tcl_Interp* interp,
+	      int argc, char** argv)
 {
   int hand;
   int idx ;
@@ -936,11 +942,8 @@ Msqltcl_Next (clientData, interp, argc, argv)
  *	number of remaining rows
  */
 
-Msqltcl_Seek (clientData, interp, argc, argv)
-    ClientData   clientData;
-    Tcl_Interp  *interp;
-    int          argc;
-    char       **argv;
+Msqltcl_Seek (ClientData clientData, Tcl_Interp* interp,
+	      int argc, char** argv)
 {
     int hand;
     int res;
@@ -998,11 +1001,8 @@ Msqltcl_Seek (clientData, interp, argc, argv)
  */
 
 int
-Msqltcl_Map (clientData, interp, argc, argv)
-     ClientData   clientData;
-     Tcl_Interp  *interp;
-     int          argc;
-     char       **argv;
+Msqltcl_Map (ClientData clientData, Tcl_Interp* interp,
+	     int argc, char** argv)
 {
   int code ;
   int count ;
@@ -1090,11 +1090,8 @@ Msqltcl_Map (clientData, interp, argc, argv)
  */
 
 int
-Msqltcl_Info (clientData, interp, argc, argv)
-     ClientData   clientData;
-     Tcl_Interp  *interp;
-     int          argc;
-     char       **argv;
+Msqltcl_Info (ClientData clientData, Tcl_Interp* interp,
+		 int argc, char** argv)
 {
   char buf[MSQL_BUFF_SIZE];
   int count ;
@@ -1193,11 +1190,8 @@ Msqltcl_Info (clientData, interp, argc, argv)
  */
 
 int
-Msqltcl_Result (clientData, interp, argc, argv)
-     ClientData   clientData;
-     Tcl_Interp  *interp;
-     int          argc;
-     char       **argv;
+Msqltcl_Result (ClientData clientData, Tcl_Interp* interp,
+		int argc, char** argv)
 {
   char buf[MSQL_BUFF_SIZE];
   int count ;
@@ -1256,7 +1250,7 @@ Msqltcl_Result (clientData, interp, argc, argv)
     case MSQL_RESCURQ_OPT:
       sprintf (interp->result, "%d", msqlNumRows (MsqlHandle[hand].result)
 	       - MsqlHandle[hand].res_count) ;
-    default:
+    default: ;
     }
   return TCL_OK ;
 }
@@ -1278,11 +1272,8 @@ Msqltcl_Result (clientData, interp, argc, argv)
  * SIDE EFFECT: '-current' disturbs the field position of the result.
  */
 
-Msqltcl_Col (clientData, interp, argc, argv)
-     ClientData   clientData;
-     Tcl_Interp  *interp;
-     int          argc;
-     char       **argv;
+Msqltcl_Col (ClientData clientData, Tcl_Interp* interp,
+	     int argc, char** argv)
 {
   char buf[MSQL_BUFF_SIZE];
   int coln ;
@@ -1393,11 +1384,8 @@ Msqltcl_Col (clientData, interp, argc, argv)
  *	                
  */
 
-Msqltcl_State (clientData, interp, argc, argv)
-     ClientData   clientData;
-     Tcl_Interp  *interp;
-     int          argc;
-     char       **argv;
+Msqltcl_State (ClientData clientData, Tcl_Interp* interp,
+	       int argc, char** argv)
 {
   int hi;
   char* hp ;
@@ -1439,11 +1427,8 @@ Msqltcl_State (clientData, interp, argc, argv)
  *	null string
  */
 
-Msqltcl_Close (clientData, interp, argc, argv)
-     ClientData   clientData;
-     Tcl_Interp  *interp;
-     int          argc;
-     char       **argv;
+Msqltcl_Close (ClientData clientData, Tcl_Interp* interp,
+	       int argc, char** argv)
 {
   int     hand;
   
